<!-- #include virtual="/quickstart/howto/include/wfheader.inc" -->

<h4>Adding an error handler for untrapped exceptions</h4>

<P>In your application design you may want to register an exception 
handler for any exceptions you have not handled in your application.  You 
can do create an exception handler for your unhandled exceptions by 
calling the <B>Application.AddOnThreadException</B> method and passing in 
a reference to a method in your application.  This method will then be 
called for any unhandled exceptions on that thread.  The 
<B>OnThreadException</B> event is raised when the Win Forms window 
procedure receives an exception.</P> 
      
<P>Win Forms provides a default exception handler and exception handling 
dialog named <B>ThreadExceptionDialog</B>.  This dialog works is typically 
used during development, but you will want to supply your own custom 
exception handlers for your application when it is near completion.</P> 

<P>The following example demonstrates how to create a very simple custom 
exception handler that displays the exception message and stack trace in a 
<b>MessageBox</b>:</P> 

<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
    <pre class=code1>
    
    <span class=codecomment1>//The Error Handler class</span>
    <span class=codecomment1>//We need a class because event handling methods can't be static</span>
    internal class CustomExceptionHandler {

        <span class=codecomment1>//Handle the exception event</span>
        public void OnThreadException(object sender, ThreadExceptionEventArgs t) {

            DialogResult result = DialogResult.Cancel;
            try {
                result = this.ShowThreadExceptionDialog(t.exception);
            }
            catch {
                try {
                    MessageBox.Show("Fatal Error", "Fatal Error", MessageBox.IconStop|MessageBox.AbortRetryIgnore);
                }
                finally {
                    Application.Exit();
                }
            }

            if (result == DialogResult.Abort) 
                Application.Exit();

        }

        <span class=codecomment1>//The simple dialog that is displayed when this class catches and exception</span>
        private DialogResult ShowThreadExceptionDialog(Exception e) {
            string errorMsg = "An error occurred please contact the adminstrator with the following information:\n\n";
            errorMsg = errorMsg + e.Message + "\n\nStack Trace:\n" + e.StackTrace;
            return MessageBox.Show(errorMsg, "Application Error", MessageBox.IconStop|MessageBox.AbortRetryIgnore);
        }
    }

    ....
    
    <span class=codecomment1>//Register the custom error handler as soon as we can in Main</span>
    <span class=codecomment1>//to make sure that we catch as many exceptions as possible</span>
    public static void Main(string[] args) {
        CustomExceptionHandler eh = new CustomExceptionHandler();
        Application.AddOnThreadException(new ThreadExceptionEventHandler(eh.OnThreadException));
        Application.Run(new ErrorHandler());
    }
    </pre>
    </template>
    
    <template name="VBTemplate">
    <pre class=code1>
    VB TBD
    </pre>
    </template>
    
</Code:CodeSnippet>

<br><Br>

<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/howto/Samples/WinForms/Cs/ErrorHandler" 
        ViewSource="/quickstart/howto/Samples/WinForms/Cs/ErrorHandler/ErrorHandler.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Error Handling Sample"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/howto/Samples/WinForms/VB/ErrorHandler" 
        ViewSource="/quickstart/howto/Samples/WinForms/VB/ErrorHandler/ErrorHandler.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Error Handling Sample"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>


<!-- #include virtual="/quickstart/howto/include/footer.inc" -->
