<!-- #include virtual="/quickstart/howto/include/wfheader.inc" -->

<h4>Creating an Owner-Drawn Listbox</h4>

      <P>By default Listboxes display strings. If you want to display something 
      else,&nbsp; such a bitmap for each item or paint each item in the 
      list box in a different color, then you will need to draw each menu item in your code. This is known as an owner-drawn
      ListBox.</P>
      <P>In order to put a ListBox into owner-drawn mode you must set the <b> 
      DrawMode</b> property to <B>DrawMode.OwnerDrawVariable</B> or 
      <B>DrawMode.OwnerDrawFixed</B>. OwnerDrawFixed indicates that the ListBox 
      will be owner-drawn and each item in the ListBox will be the same height. 
      OwnerDrawVariable indicates that the ListBox will be owner-drawn and each 
      item in the ListBox may be a different height. 
      <P>There are two events that you need to handle for an owner-drawn list 
      box:</P>
      <TABLE border=0 cellPadding=5 class=mydd>
        <TBODY>
        <TR>
          <TD vAlign=top><STRONG>DrawItem</STRONG></TD>
          <TD>
            <P>This event is raised every time the list box needs to paint an 
            item. The handler for this event is passed a <B>DrawItemEventArgs</B> 
            object. This object contains the information needed to draw the 
            ListBox item including a <b> Graphics</b> object, the index of the item to 
            be painted, the state of the item to be painted, and the bounds of the 
            item to be painted. It also includes utility methods for 
            painting the background of the item and its focus 
        rectangle.</P>
          </TD></TR>
        <TR>
          <TD vAlign=top><STRONG>MeasureItem</STRONG></TD>
          <TD>
            <P>This event is raised every time a ListBox item is created and the 
            size of the list item need to be determined. You only need to handle 
            this event if the Listbox DrawMode is <B>OwnerDrawVariable</B>. The handler for this
            event is passed a 
            <B>MeasureItemEventArgs</B> object. This object contains the 
            information needed to determine the size of the item,&nbsp; a Graphics
            object, and the index of the item to be measured. To set the height
            and width of each item use the ItemHeight and ItemWidth properties on the MeasureItemEventArgs 
            object.</TD></TR></TBODY></TABLE>
      <P>The following example demonstrates how to create an owner-drawn ListBox that displays 
      each item in the ListBox in a different color:</P>
      
<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <pre class=code1>

    ....    
    
    <span class=codecomment1>//Set the DrawMode and add the event handlers</span>
    listBox1.DrawMode=DrawMode.OwnerDrawVariable;
    listBox1.AddOnMeasureItem(new MeasureItemEventHandler(listBox1_MeasureItem));
    wlistBox1.AddOnDrawItem(new DrawItemEventHandler(listBox1_DrawItem));

    ....    

    <span class=codecomment1>//The MeasureItem Event handler</span>
    private void listBox1_MeasureItem(object sender, MeasureItemEventArgs e) {
            
        <span class=codecomment1>//Work out what the text will be</span>
        string displayText = "ITEM #" + e.Index;

        <span class=codecomment1>//Get width & height of string</span>
        SizeF stringSize=e.Graphics.MeasureString(displayText, this.Font);

        <span class=codecomment1>//Now set height to taller of default and text height</span>
        if (listBoxHeights[e.Index] > stringSize.Height)
            e.ItemHeight = listBoxHeights[e.Index];
        else 
            e.ItemHeight = (int)stringSize.Height;

    }

    ....
    
    <span class=codecomment1>//The DrawItem Event handler</span>
    private void listBox1_DrawItem(object sender, DrawItemEventArgs e) {
        
        <span class=codecomment1>// The following method should generally be called before drawing.</span>
        <span class=codecomment1>// It is actually superfluous here, since the subsequent drawing</span>
        <span class=codecomment1>// will completely cover the area of interest.</span>
        e.DrawBackground();
        
        <span class=codecomment1>//The system provides the Graphics object</span>
        <span class=codecomment1>//into which the list box item is drawn.</span>
        <span class=codecomment1>//The index of the item to be painted is e.Index.</span>
        <span class=codecomment1>//The painting should be done into the area described by e.Bounds.</span>
        Brush brush = listBoxBrushes[e.Index];
        e.Graphics.FillRectangle(brush, e.Bounds);
        e.Graphics.DrawRectangle(SystemPens.WindowText, e.Bounds);
        
        bool selected = ((e.State & DrawItemState.Selected) == DrawItemState.Selected) ? true : false;
            
        string displayText = "ITEM #" + e.Index;
        displayText = displayText + (selected ? " SELECTED" : "");

        e.Graphics.DrawString(displayText, this.Font, Brushes.Black,e.Bounds);

        <span class=codecomment1>//Finally draw the focus rectangle for the item</span>
        e.DrawFocusRectangle();
    }

    ....

    </pre>
</template>

<template name="VBTemplate">
   <pre class=code1>
   </pre>
</template>
</Code:CodeSnippet>

<br><br>

<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/howto/Samples/WinForms/Cs/OwnerDrawListBox" 
        ViewSource="/quickstart/howto/Samples/WinForms/Cs/OwnerDrawListBox/OwnerDrawListBox.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# OwnerDraw ListBox Sample"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/howto/Samples/WinForms/VB/OwnerDrawListBox" 
        ViewSource="/quickstart/howto/Samples/WinForms/VB/OwnerDrawListBox/OwnerDrawListBox.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB OwnerDraw ListBox Sample"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>


<!-- #include virtual="/quickstart/howto/include/footer.inc" -->
