<!-- #include virtual="/quickstart/howto/include/wfheader.inc" -->

<h4>Making procedure calls across thread boundaries</h4>

<P>WinForms controls can only execute on the thread on which they were 
created - they are not thread safe.  If you want to get or set properties 
or call methods on a control from a background thread the call must be 
marshaled to the thread that created the control.</P> 
      
<P>There are five functions on a control that are safe to call from any 
thread: <b>InvokeRequired</b>, <b>Invoke</b>, <b>BeginInvoke</b>, <b> 
EndInvoke</b> and <b>CreateGraphics</b>.  For all other method calls, you 
should use one of the invoke methods.</P> 
      
<P>By default Windows marshals the calls for you.  However if you are 
making multiple calls to a control it is much more efficient to create a 
method that executes those calls and make that one cross-thread call 
yourself.  You make the cross thread call by calling one of the <b> 
Control.Invoke</b> methods.  The Invoke methods take a reference to a 
delegate - typically this delegate is an instance of the 
<B>MethodInvoker</B> delegate.</P> 
      
      <TABLE border=0 cellPadding=5 class=mydd>
        <TBODY>
        <TR>
          <TD vAlign=top><STRONG>InvokeRequired</STRONG></TD>
          <TD>
            <P>public bool InvokeRequired { get ; }</P>
            <P>Returns true if the caller must call Invoke when making method 
            calls to this control.</P></TD></TR>
        <TR>
          <TD vAlign=top><STRONG>BeginInvoke</STRONG></TD>
          <TD>
            <P>public IAsyncResult BeginInvoke(Delegate method)<BR>public 
            IAsyncResult BeginInvoke(Delegate method, Object[] args)</P>
            <P>Executes the given delegate on the thread that owns this 
            Control's underlying window handle. The delegate is called 
            asynchronously and this method returns immediately. You may call 
            this from any thread, even the thread that owns the control's 
            handle. If the control's handle doesn't exist yet, this will follow 
            up the control's parent chain until it finds a control or form that 
            does have a window handle. If no appropriate handle can be found, 
            BeginInvoke will throw an exception. Exceptions within the delegate 
            method are considered untrapped and will be sent to the 
            application's untrapped exception handler. </P></TD></TR>
        <TR>
          <TD vAlign=top><STRONG>EndInvoke</STRONG></TD>
          <TD>
            <P>public Object EndInvoke(IAsyncResult 
            asyncResult)<BR></P>Retrieves the return value of the asynchronous 
            operation represented by the IAsyncResult interface passed. If the 
            async operation has not been completed, this function will block 
            until the result is available. 
            <P></P></TD></TR>
        <TR>
          <TD vAlign=top><STRONG>Invoke</STRONG></TD>
          <TD>
            <P>public Object Invoke(Delegate method)<BR>public Object 
            Invoke(Delegate method, Object[] args)<BR>
            <P>Executes the given delegate on the thread that owns this 
            Control's underlying window handle. The delegate is called 
            synchronously and this method returns once the invoked method has 
            returned. The return value is the result of the invoked method. It 
            is an error to call this on the same thread that the control belongs 
            to. </P></TD></TR></TBODY></TABLE><BR>
      <P>The following example demonstrates how to create a background thread that uses a
      <b> 
      MethodInvoker</b> to update a <b>ProgressBar</b> control at regular
      intervals:</P>
      

<Code:CodeSnippet runat=server>

<template name="CsTemplate">
    <pre class=code1>

    ....    
    
    <span class=codecomment1>//Start the background thread</span>
    timerThread = new Thread(new ThreadStart(ThreadProc));
    timerThread.IsBackground = true;
    timerThread.Start();
    
    ....    
    
    <span class=codecomment1>//This function is executed on a background thread - it marshalls calls to</span>
    <span class=codecomment1>//update the UI back to the foreground thread</span>
    public void ThreadProc() {
        
        try {
            MethodInvoker mi = new MethodInvoker(this.UpdateProgress);
            while (true) {
            
                <span class=codecomment1>//Call BeginInvoke on the Form</span>
                this.BeginInvoke(mi);
                Thread.Sleep(500) ;
            }
        }
        <span class=codecomment1>//Thrown when the thread is interupted by the main thread - exiting the loop</span>
        catch (ThreadInterruptedException e) {
            <span class=codecomment1>//Simply exit....</span>
        }
        catch (Exception we) {
        }
    }
    
    ....
    
    <span class=codecomment1>//This function is called from the background thread</span>
    private void UpdateProgress() {

        <span class=codecomment1>//Reset to start if required</span>
        if (progressBar1.Value == progressBar1.Maximum) {
            progressBar1.Value = progressBar1.Minimum ;
        }


        <span class=codecomment1>//Reset to start if required</span>
        progressBar1.PerformStep() ;
    }

    ....
 
    <span class=codecomment1>//Make sure we clean up the background thread in Dispose</span>
    public override void Dispose() {
       if (timerThread != null) {
           timerThread.Interrupt();
           timerThread = null;
       }

       base.Dispose();
   }

    </pre>
</template>

<template name="VBTemplate">
   <pre class=code1>
   </pre>
</template>
</Code:CodeSnippet>

<br><br>

<Code:CodeSnippet runat=server>
    
    <template name="CsTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/howto/Samples/WinForms/Cs/ThreadMarshal" 
        ViewSource="/quickstart/howto/Samples/WinForms/Cs/ThreadMarshal/ThreadMarshal.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="C# Threading Sample"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
    <template name="VBTemplate">
        <Acme:SourceRef 
        RunSample="/quickstart/howto/Samples/WinForms/VB/ThreadMarshal" 
        ViewSource="/quickstart/howto/Samples/WinForms/VB/ThreadMarshal/ThreadMarshal.src"
        Icon = "/quickstart/winforms/images/wflink.jpg"
        Caption="VB Threading Sample"
        MapRunSamplePath=true
        runat="server" />
    </template>
    
</Code:CodeSnippet>



<!-- #include virtual="/quickstart/howto/include/footer.inc" -->
